package com.jivosite.jivosdk_plugin

import android.content.Context

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import android.content.Intent
import android.util.Log
import com.google.firebase.messaging.RemoteMessage
import com.jivosite.sdk.ui.chat.JivoChatActivity
import com.jivosite.sdk.Jivo
import timber.log.Timber
import com.jivosite.sdk.support.builders.ContactInfo
import com.jivosite.sdk.model.pojo.CustomData
import io.flutter.plugin.common.MethodChannel.Result


/** JivoPlatform */
class JivoPlugin : FlutterPlugin, MethodCallHandler {

    private lateinit var applicationContext: Context
    private lateinit var channel: MethodChannel


    override fun onAttachedToEngine(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        applicationContext = flutterPluginBinding.applicationContext
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "jivosdk_plugin")
        channel.setMethodCallHandler(this)
        Timber.plant(Timber.DebugTree())
        Jivo.init(applicationContext)
        Jivo.enableLogging()
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "session:setup" -> {
                val data = call.arguments as Map<String, String>
                val widgetId = data["channel_id"] ?: ""
                val userToken = data["userToken"] ?: ""
                if (widgetId.contains('/')) {
                    val list = widgetId.split('/')
                    Jivo.setData(widgetId = list.last(), userToken = userToken, host = list.first())
                } else {
                    Jivo.setData(widgetId = widgetId, userToken = userToken)
                }
            }

            "notification:setPushToken" -> {
                val token = call.arguments as String
                if (!token.isNullOrBlank()) {
                    Jivo.updatePushToken(call.arguments as String)
                }
            }

            "session:setContactInfo" -> {
                val contactInfo = call.arguments as Map<String, String>
                Jivo.setContactInfo(ContactInfo.contactInfo {
                    name = contactInfo["name"]
                    email = contactInfo["email"] ?: ""
                    phone = contactInfo["phone"] ?: ""
                    description = contactInfo["brief"] ?: ""
                })
            }

            "session:setCustomData" -> {
                val fields = call.arguments as List<Map<String, String>>
                Jivo.setCustomData(fields.map {
                    CustomData(
                        content = it["content"] ?: "", title = it["title"] ?: "", link = it["link"] ?: "", key = it["key"] ?: ""
                    )
                })
            }

            "display:present" -> {
                Intent(applicationContext, JivoChatActivity::class.java).apply {
                    if (resolveActivity(applicationContext.packageManager) != null) {
                        flags = Intent.FLAG_ACTIVITY_NEW_TASK
                        applicationContext.startActivity(this)
                    }
                }
                result.success(null)
            }

            "session:clear" -> {
                Jivo.clear()
            }

            else -> result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }
}
